@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: examp_c.c,v 1.15 2012/06/28 16:54:53 diaz Exp $ */
+/* $Id$ */
 
 #include <string.h>
 
@@ -217,7 +217,11 @@ my_call(PlTerm goal)
   if (result == PL_EXCEPTION)
     {
       PlTerm except = Pl_Get_Exception();
+#if 0
       Pl_Exec_Continuation(Pl_Find_Atom("throw"), 1, &except);
+#else
+      Pl_Throw(except);
+#endif
     }
 
   return result;
