@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_bool_c.c,v 1.21 2012/03/15 09:39:42 diaz Exp $ */
+/* $Id$ */
 
 #define OBJ_INIT Fd_Bool_Initializer
 
@@ -106,7 +106,7 @@ static WamWord *sp;
 static WamWord vars_tbl[VARS_STACK_SIZE];
 static WamWord *vars_sp;
 
-static Bool (*func_tbl[NB_OF_OP + 2]) ();
+static Bool (*func_tbl[NB_OF_OP + 2]) (WamWord *exp, int result, WamWord *load_word);
 
 
 
@@ -285,19 +285,20 @@ Pl_Fd_Bool_Meta_3(WamWord le_word, WamWord re_word, WamWord op_word)
   WamWord *adr, *fdv_adr;
   WamWord *exp;
   int op;
+  static WamWord h[3];		/* static to avoid high address */
 
 
   DEREF(op_word, word, tag_mask);
   op = UnTag_INT(op_word);
 
-  H[0] = bool_tbl[op];		/* also works for NOT/1 */
-  H[1] = le_word;
-  H[2] = re_word;
+  h[0] = bool_tbl[op];		/* also works for NOT/1 */
+  h[1] = le_word;
+  h[2] = re_word;
 
   sp = stack;
   vars_sp = vars_tbl;
 
-  exp = Simplify(1, Tag_STC(H));
+  exp = Simplify(1, Tag_STC(h));
 
 #ifdef DEBUG
   Display_Stack(exp);
@@ -382,7 +383,7 @@ Simplify(int sign, WamWord e_word)
 
 #ifdef DEBUG
   printf("ENTERING %5ld: %2d: ", sp - stack, sign);
-  Pl_Write_Simple(e_word);
+  Pl_Write(e_word);
   printf("\n");
 #endif
 
@@ -1043,7 +1044,7 @@ Set_Not(WamWord *exp, int result, WamWord *load_word)
 
 
 /*-------------------------------------------------------------------------*
- * SET_EQUIV                                                               *
+ * SET_EQUIV                                                               *<
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static Bool
@@ -1575,18 +1576,26 @@ Pl_Fd_Reified_In(WamWord x_word, WamWord l_word, WamWord u_word, WamWord b_word)
   WamWord word, tag_mask;
   WamWord b_tag_mask, x_tag_mask;
   WamWord *adr, *fdv_adr;
-  int x;
-  int l = Pl_Rd_Integer_Check(l_word);
-  int u = Pl_Rd_Integer_Check(u_word);
-  int b = -1;			/* a var */
-  Range *r;
+  PlLong x;
+  PlLong b = -1;		/* a var */
+  int min, max;
   int x_min, x_max;
+  Range *r;
  
+  //  Bool pl_fd_domain(WamWord x_word, WamWord l_word, WamWord u_word);
+  /* from fd_values_c.c (optimized version) */
+  Bool Pl_Fd_Domain_Interval(WamWord x_word, int min, int max);
 
-  Bool pl_fd_domain(WamWord x_word, WamWord l_word, WamWord u_word);
+  /* from fd_values_fd.fd */
   Bool pl_fd_not_domain(WamWord x_word, WamWord l_word, WamWord u_word);
 
 
+  min = Pl_Fd_Prolog_To_Value(l_word);
+  if (min < 0)
+    min = 0;
+  max = Pl_Fd_Prolog_To_Value(u_word);
+
+
   DEREF(x_word, word, tag_mask);
   x_word = word;
   x_tag_mask = tag_mask;
@@ -1607,7 +1616,7 @@ Pl_Fd_Reified_In(WamWord x_word, WamWord l_word, WamWord u_word, WamWord b_word)
   if (x_tag_mask == TAG_INT_MASK)
     {
       x = UnTag_INT(x_word);
-      b = (x >= l) && (x <= u);
+      b = (x >= min) && (x <= max);
     unif_b:
       return Pl_Get_Integer(b, b_word);
     }
@@ -1617,7 +1626,7 @@ Pl_Fd_Reified_In(WamWord x_word, WamWord l_word, WamWord u_word, WamWord b_word)
       b = UnTag_INT(b_word);
       if (b == 0)
 	return pl_fd_not_domain(x_word, l_word, u_word);
-      return (b == 1) && pl_fd_domain(x_word, l_word, u_word);
+      return (b == 1) && Pl_Fd_Domain_Interval(x_word, min, max);
     }
 
 
@@ -1635,13 +1644,13 @@ Pl_Fd_Reified_In(WamWord x_word, WamWord l_word, WamWord u_word, WamWord b_word)
   x_min = r->min;
   x_max = r->max;
 
-  if (x_min >= l && x_max <= u)
+  if (x_min >= min && x_max <= max)
     {
       b = 1;
       goto unif_b;
     }
 
-  if (l > u || x_max < l || x_min > u) /* NB: if L..U is empty then B = 0 */
+  if (min > max || x_max < min || x_min > max) /* NB: if L..U is empty then B = 0 */
     {
       b = 0;
       goto unif_b;
