@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_values_c.c,v 1.21 2012/03/15 09:39:42 diaz Exp $ */
+/* $Id$ */
 
 #include <stdlib.h>
 
@@ -54,9 +54,15 @@
 
 #define METHOD_MIN                 0
 #define METHOD_MAX                 1
-#define METHOD_MIDDLE              2
-#define METHOD_LIMITS              3
-#define METHOD_RANDOM_V            4
+#define METHOD_RANDOM_V            2
+#define METHOD_MIDDLE              3
+#define METHOD_BISECT              4
+#define METHOD_LIMITS              5
+
+#define METHOD_LIMITS_MIN          METHOD_LIMITS
+#define METHOD_LIMITS_MAX          (METHOD_LIMITS + 1)
+
+
 
 #define METHOD_STANDARD            0
 #define METHOD_FIRST_FAIL          1
@@ -86,18 +92,6 @@ typedef Bool (*CmpFct) ();
  * Function Prototypes             *
  *---------------------------------*/
 
-static Bool Indomain_Min(WamWord *fdv_adr);
-
-static Bool Indomain_Max(WamWord *fdv_adr);
-
-static Bool Indomain_Middle(WamWord *fdv_adr);
-
-static Bool Indomain_Limits(WamWord *fdv_adr);
-
-static Bool Indomain_Random(WamWord *fdv_adr);
-
-
-
 static Bool Cmp_First_Fail(WamWord *last_fdv_adr, WamWord *new_fdv_adr);
 
 static Bool Cmp_Most_Constrained(WamWord *last_fdv_adr, WamWord *new_fdv_adr);
@@ -110,80 +104,22 @@ static Bool Cmp_Max_Regret(WamWord *last_fdv_adr, WamWord *new_fdv_adr);
 
 
 
-#define INDOMAIN_MIN_ALT           X1_24696E646F6D61696E5F6D696E5F616C74
-
-#define INDOMAIN_MAX_ALT           X1_24696E646F6D61696E5F6D61785F616C74
-
-#define INDOMAIN_MIDDLE_ALT        X1_24696E646F6D61696E5F6D6964646C655F616C74
-
-#define INDOMAIN_LIMITS_ALT        X1_24696E646F6D61696E5F6C696D6974735F616C74
-
-#define INDOMAIN_RANDOM_ALT        X1_24696E646F6D61696E5F72616E646F6D5F616C74
+#define INDOMAIN_ALT               X1_24696E646F6D61696E5F616C74
 
 #define EXTRA_CSTR_ALT             X1_2465787472615F637374725F616C74
 
+Prolog_Prototype(INDOMAIN_ALT, 0);
 
-
-Prolog_Prototype(INDOMAIN_MIN_ALT, 0);
-Prolog_Prototype(INDOMAIN_MAX_ALT, 0);
-Prolog_Prototype(INDOMAIN_MIDDLE_ALT, 0);
-Prolog_Prototype(INDOMAIN_LIMITS_ALT, 0);
-Prolog_Prototype(INDOMAIN_RANDOM_ALT, 0);
 Prolog_Prototype(EXTRA_CSTR_ALT, 0);
 
+
   /* defined in fd_values_fd.fd */
 
-Bool pl_fd_domain(WamWord x_word, WamWord l_word, WamWord u_word);
-
 Bool pl_fd_domain_r(WamWord x_word, WamWord r_word);
 
 
 
 
-/*-------------------------------------------------------------------------*
- * PL_FD_DOMAIN_BOOL_1                                                     *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-Bool
-Pl_Fd_Domain_Bool_1(WamWord list_word)
-{
-  WamWord word, tag_mask;
-  WamWord save_list_word;
-  WamWord *lst_adr;
-
-
-  DEREF(list_word, word, tag_mask);
-  if (tag_mask == TAG_REF_MASK || tag_mask == TAG_INT_MASK ||
-      tag_mask == TAG_FDV_MASK)
-    return Pl_Fd_Check_For_Bool_Var(word);
-
-  save_list_word = list_word;
-  for (;;)
-    {
-      DEREF(list_word, word, tag_mask);
-
-      if (tag_mask == TAG_REF_MASK)
-	Pl_Err_Instantiation();
-
-      if (word == NIL_WORD)
-	break;
-
-      if (tag_mask != TAG_LST_MASK)
-	Pl_Err_Type(pl_type_list, save_list_word);
-
-      lst_adr = UnTag_LST(word);
-      if (!Pl_Fd_Check_For_Bool_Var(Car(lst_adr)))
-	return FALSE;
-
-      list_word = Cdr(lst_adr);
-    }
-
-  return TRUE;
-}
-
-
-
-
 /*-------------------------------------------------------------------------*
  * PL_FD_DOMAIN_2                                                          *
  *                                                                         *
@@ -197,8 +133,7 @@ Pl_Fd_Domain_2(WamWord list_word, WamWord r_word)
 
 
   DEREF(list_word, word, tag_mask);
-  if (tag_mask == TAG_REF_MASK || tag_mask == TAG_INT_MASK ||
-      tag_mask == TAG_FDV_MASK)
+  if (tag_mask == TAG_REF_MASK || tag_mask == TAG_INT_MASK || tag_mask == TAG_FDV_MASK)
     return pl_fd_domain_r(word, r_word);
 
   save_list_word = list_word;
@@ -227,6 +162,84 @@ Pl_Fd_Domain_2(WamWord list_word, WamWord r_word)
 
 
 
+/* domain(X,L,U) is optimized here.
+ * Previously it called pl_fd_domain (defined in fd_values_fd.fd as X in L..U)
+ * This version avoids AFrame creation and FD var creation if interval is empty or an INT
+ *
+ * We do not do the same for domain(X,L) (which continues to call pl_fd_domain_r)
+ * Because we have to be cautious when handling range directly (see Pl_Tell_Range_Range
+ * which recovers CS/save_CS). Also to avoid to handle extra cstr warning messages
+ * (for instance when a range is empty or a singleton and extr_cstr is TRUE).
+ *
+ * The 2 following functions could be moved to EngineFD/fd_inst.c (is often use): 
+ *    Pl_Fd_Domain_Interval 
+ *    Pl_Fd_Domain_Var_3  (renamed as Pl_Fd_Prolog_Domain)
+ * 
+ */
+
+
+/*-------------------------------------------------------------------------*
+ * Pl_FD_DOMAIN_INTERVAL                                                   *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_Domain_Interval(WamWord x_word, int min, int max)
+{
+  WamWord word, tag_mask;
+  WamWord *adr, *fdv_adr;
+  PlLong v;
+
+  DEREF(x_word, word, tag_mask);
+
+  if (tag_mask == TAG_REF_MASK)
+    {
+      if (min > max)
+	return FALSE;
+
+      if (min == max)
+	return Pl_Get_Integer(min, x_word);
+
+      adr = UnTag_REF(word);
+      fdv_adr = Pl_Fd_New_Variable_Interval(min, max);
+      Bind_UV(adr, Tag_REF(fdv_adr));
+      return TRUE;
+    }
+
+  if (tag_mask == TAG_INT_MASK)
+    {
+      v = UnTag_INT(word);
+      return (v >= min && v <= max); /* also detects if min > max */
+    }
+
+  if (tag_mask != TAG_FDV_MASK)
+    Pl_Err_Type(pl_type_fd_variable, word);
+
+  return Pl_Fd_In_Interval(UnTag_FDV(word), min, max);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_FD_DOMAIN_VAR_3                                                      *
+ *                                                                         *
+ * Only accepts a var (not a list)                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_Domain_Var_3(WamWord x_word, WamWord l_word, WamWord u_word)
+{
+  int min, max;
+
+  min = Pl_Fd_Prolog_To_Value(l_word);
+  if (min < 0)
+    min = 0;
+  max = Pl_Fd_Prolog_To_Value(u_word);
+
+  return Pl_Fd_Domain_Interval(x_word, min, max);  
+}
+
+
+
 
 /*-------------------------------------------------------------------------*
  * PL_FD_DOMAIN_3                                                          *
@@ -238,14 +251,16 @@ Pl_Fd_Domain_3(WamWord list_word, WamWord l_word, WamWord u_word)
   WamWord word, tag_mask;
   WamWord save_list_word;
   WamWord *lst_adr;
+  int min, max;
 
-
-  if (l_word == Tag_INT(0) && u_word == Tag_INT(1))
-    return Pl_Fd_Domain_Bool_1(list_word);
+  min = Pl_Fd_Prolog_To_Value(l_word);
+  if (min < 0)
+    min = 0;
+  max = Pl_Fd_Prolog_To_Value(u_word);
 
   DEREF(list_word, word, tag_mask);
   if (tag_mask == TAG_REF_MASK || tag_mask == TAG_INT_MASK || tag_mask == TAG_FDV_MASK)
-    return pl_fd_domain(word, l_word, u_word);
+    return Pl_Fd_Domain_Interval(word, min, max);
 
   save_list_word = list_word;
   for (;;)
@@ -262,7 +277,7 @@ Pl_Fd_Domain_3(WamWord list_word, WamWord l_word, WamWord u_word)
 	Pl_Err_Type(pl_type_list, save_list_word);
 
       lst_adr = UnTag_LST(word);
-      if (!pl_fd_domain(Car(lst_adr), l_word, u_word))
+      if (!Pl_Fd_Domain_Interval(Car(lst_adr), min, max))
 	return FALSE;
 
       list_word = Cdr(lst_adr);
@@ -274,459 +289,154 @@ Pl_Fd_Domain_3(WamWord list_word, WamWord l_word, WamWord u_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * SELECT_VALUE                                                            *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static int
+Select_Value(WamWord *fdv_adr, int value_method)
+{
+  int n;
+  switch(value_method)
+    {
+    case METHOD_MIN:
+    case METHOD_LIMITS_MIN:
+      return Min(fdv_adr);
+
+    case METHOD_MAX:
+    case METHOD_LIMITS_MAX:
+      return Max(fdv_adr);
+
+    case METHOD_BISECT:
+    case METHOD_MIDDLE:
+      n = Nb_Elem(fdv_adr) / 2;			   /* here nb_elem > 1 => n >= 1 */
+      return Pl_Range_Ith_Elem(Range(fdv_adr), n); /* Ith is in 1..nb_elem */
+      
+    case METHOD_RANDOM_V:
+      n = Nb_Elem(fdv_adr);
+      n = Pl_M_Random_Integer(n);		       /* random returns in 0..nb_elem-1 */
+      return Pl_Range_Ith_Elem(Range(fdv_adr), n + 1); /* Ith is in 1..nb_elem */
+    }
+
+  return 0;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_INDOMAIN_2                                                           *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Indomain_2(WamWord fdv_word, WamWord method_word)
+Pl_Indomain_2(WamWord x_word, WamWord method_word)
 {
   WamWord word, tag_mask;
   WamWord *fdv_adr;
+  int value_method;
+  int value;
 
-  Fd_Deref_Check_Fd_Var(fdv_word, word, tag_mask);
+  value_method = Pl_Rd_Integer(method_word);
 
-  if (tag_mask == TAG_INT_MASK)
-    return TRUE;
+  Fd_Deref_Check_Fd_Var(x_word, word, tag_mask);
 
   fdv_adr = UnTag_FDV(word);
 
-  switch (Pl_Rd_Integer_Check(method_word))
+ bisect_terminal_rec:
+  if (tag_mask == TAG_INT_MASK)
+    return TRUE;
+
+  value = Select_Value(fdv_adr, value_method);
+  
+  A(0) = (WamWord) fdv_adr | Extra_Cstr(fdv_adr);
+  A(1) = value_method;
+  A(2) = value;
+
+  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_ALT, 0), 3);
+
+  if (value_method == METHOD_BISECT)
     {
-    case METHOD_MIN:
-      return Indomain_Min(fdv_adr);
+      if (!Pl_Fd_In_Interval(fdv_adr, 0, value))
+	return FALSE;
 
-    case METHOD_MAX:
-      return Indomain_Max(fdv_adr);
+      tag_mask = Tag_Mask_Of(*fdv_adr);
 
-    case METHOD_MIDDLE:
-      return Indomain_Middle(fdv_adr);
-
-    case METHOD_LIMITS:
-      return Indomain_Limits(fdv_adr);
-
-    case METHOD_RANDOM_V:
-      return Indomain_Random(fdv_adr);
+      goto bisect_terminal_rec;
     }
 
-  return TRUE;
+  return Pl_Fd_Assign_Value_Fast(fdv_adr, value);
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * INDOMAIN_MIN                                                            *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static Bool
-Indomain_Min(WamWord *fdv_adr)
-{
-  Range *range;
-  int cur, end;
-
-  range = Range(fdv_adr);
-
-  end = Max(fdv_adr);
-  cur = Min(fdv_adr);
-
-  A(0) = (WamWord) fdv_adr;
-  A(1) = (WamWord) range;
-  A(2) = end;
-  A(3) = cur;
-
-  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_MIN_ALT, 0), 4);
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_INDOMAIN_MIN_ALT_0                                                   *
+ * PL_INDOMAIN_ALT_0                                                       *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Indomain_Min_Alt_0(void)
+Pl_Indomain_Alt_0(void)
 {
   WamWord *fdv_adr;
-  Range *range;
-  int cur, end;
+  int extra_cstr;
+  int value_method;
+  int value;
 
-  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_MIN_ALT, 0), 0);
 
-  fdv_adr = (WamWord *) AB(B, 0);
-  range = (Range *) AB(B, 1);
-  end = AB(B, 2);
-  cur = AB(B, 3);
-
-  cur = Pl_Range_Next_After(range, cur);
+  Pl_Delete_Choice_Point(3);
 
   SYS_VAR_FD_BCKTS++;
 
-  if (cur != end)
+  fdv_adr = (WamWord *) (A(0) & ~1);
+  extra_cstr = A(0) & 1;
+  value_method = A(1);
+  value = A(2);
+
+  if (value_method == METHOD_LIMITS_MIN)
+    value_method = METHOD_LIMITS_MAX;
+  else if (value_method == METHOD_LIMITS_MAX)
+    value_method = METHOD_LIMITS_MIN;
+  else if (value_method == METHOD_BISECT)
     {
-#if 0 /* the following data is unchanged */
-      AB(B, 0) = (WamWord) fdv_adr;
-      AB(B, 1) = (WamWord) range;
-      AB(B, 2) = end;
-#endif
-      AB(B, 3) = cur;
-    }
-  else if (Extra_Cstr(fdv_adr))
-    Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(EXTRA_CSTR_ALT, 0), 0);
-  else
-    Pl_Delete_Choice_Point(0);
+      /* NB: not need to test extra_cstr because it is handled by Pl_Fd_In_Interval() 
+       * (when the var becomes ground or if empty domain (failure) 
+       */
+      if (!Pl_Fd_In_Interval(fdv_adr, value + 1, INTERVAL_MAX_INTEGER))
+	return FALSE;
 
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * INDOMAIN_MAX                                                            *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static Bool
-Indomain_Max(WamWord *fdv_adr)
-{
-  Range *range;
-  int cur, end;
-
-  range = Range(fdv_adr);
-
-  end = Min(fdv_adr);
-  cur = Max(fdv_adr);
-
-  A(0) = (WamWord) fdv_adr;
-  A(1) = (WamWord) range;
-  A(2) = end;
-  A(3) = cur;
-
-  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_MAX_ALT, 0), 4);
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_INDOMAIN_MAX_ALT_0                                                   *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-Bool
-Pl_Indomain_Max_Alt_0(void)
-{
-  WamWord *fdv_adr;
-  Range *range;
-  int cur, end;
-
-  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_MAX_ALT, 0), 0);
-
-  fdv_adr = (WamWord *) AB(B, 0);
-  range = (Range *) AB(B, 1);
-  end = AB(B, 2);
-  cur = AB(B, 3);
-
-  cur = Pl_Range_Next_Before(range, cur);
-
-  SYS_VAR_FD_BCKTS++;
-
-  if (cur != end)
-    {
-#if 0 /* the following data is unchanged */
-      AB(B, 0) = (WamWord) fdv_adr;
-      AB(B, 1) = (WamWord) range;
-      AB(B, 2) = end;
-#endif
-      AB(B, 3) = cur;
-    }
-  else if (Extra_Cstr(fdv_adr))
-    Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(EXTRA_CSTR_ALT, 0), 0);
-  else
-    Pl_Delete_Choice_Point(0);
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * INDOMAIN_MIDDLE                                                         *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static Bool
-Indomain_Middle(WamWord *fdv_adr)
-{
-  Range *range;
-  int end, cur;
-  int cur_left, cur_right;
-  Bool is_right;
-  int i;
-
-  range = Range(fdv_adr);
-
-  end = Max(fdv_adr);
-  i = (Nb_Elem(fdv_adr) + 1) / 2;
-  cur = Pl_Range_Ith_Elem(range, i);
-  cur_left = cur_right = cur;
-  is_right = Nb_Elem(fdv_adr) % 2;	/* is_rigth if nb of elements is odd */
-
-  A(0) = (WamWord) fdv_adr;
-  A(1) = (WamWord) range;
-  A(2) = end;
-  A(3) = cur_left;
-  A(4) = cur_right;
-  A(5) = (WamWord) is_right;
-
-  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_MIDDLE_ALT, 0),
-		      6);
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_INDOMAIN_MIDDLE_ALT_0                                                *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-Bool
-Pl_Indomain_Middle_Alt_0(void)
-{
-  WamWord *fdv_adr;
-  Range *range;
-  int end, cur;
-  int cur_left, cur_right;
-  Bool is_right;
-
-  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_MIDDLE_ALT, 0),
-		      0);
-
-  fdv_adr = (WamWord *) AB(B, 0);
-  range = (Range *) AB(B, 1);
-  end = AB(B, 2);
-  cur_left = AB(B, 3);
-  cur_right = AB(B, 4);
-  is_right = (Bool) AB(B, 5);
-
-  if (is_right)
-    {
-      cur_left = Pl_Range_Next_Before(range, cur_left);
-      cur = cur_left;
-      is_right = FALSE;
-    }
-  else
-    {
-      cur_right = Pl_Range_Next_After(range, cur_right);
-      cur = cur_right;
-      is_right = TRUE;
+      /* simple and enough (like in Prolog) */
+      return Pl_Indomain_2(*fdv_adr, Tag_INT(value_method));
     }
 
-
-  SYS_VAR_FD_BCKTS++;
-
-  if (cur != end)
+ 
+  if (!Pl_Fd_Remove_Value(fdv_adr, value))
     {
-#if 0 /* the following data is unchanged */
-      AB(B, 0) = (WamWord) fdv_adr;
-      AB(B, 1) = (WamWord) range;
-      AB(B, 2) = end;
-#endif
-      AB(B, 3) = cur_left;
-      AB(B, 4) = cur_right;
-      AB(B, 5) = is_right;
-    }
-  else if (Extra_Cstr(fdv_adr))
-    ALTB(B) = (CodePtr) Prolog_Predicate(EXTRA_CSTR_ALT, 0);
-  else
-    Delete_Last_Choice_Point();
+      if (extra_cstr)
+	Pl_Fd_Display_Extra_Cstr(fdv_adr);
 
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * INDOMAIN_LIMITS                                                         *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static Bool
-Indomain_Limits(WamWord *fdv_adr)
-{
-  Range *range;
-  int end, cur;
-  int cur_left, cur_right;
-  Bool is_right;
-  int i;
-
-
-  range = Range(fdv_adr);
-
-  i = Nb_Elem(fdv_adr) / 2 + 1;
-  end = Pl_Range_Ith_Elem(range, i);
-  cur = Min(fdv_adr);
-  cur_left = cur;
-  cur_right = -1;
-  is_right = FALSE;
-
-  A(0) = (WamWord) fdv_adr;
-  A(1) = (WamWord) range;
-  A(2) = end;
-  A(3) = cur_left;
-  A(4) = cur_right;
-  A(5) = (WamWord) is_right;
-
-  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_LIMITS_ALT, 0),
-		      6);
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_INDOMAIN_LIMITS_ALT_0                                                *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-Bool
-Pl_Indomain_Limits_Alt_0(void)
-{
-  WamWord *fdv_adr;
-  Range *range;
-  int end, cur;
-  int cur_left, cur_right;
-  Bool is_right;
-
-  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_LIMITS_ALT, 0),
-		      0);
-
-  fdv_adr = (WamWord *) AB(B, 0);
-  range = (Range *) AB(B, 1);
-  end = AB(B, 2);
-  cur_left = AB(B, 3);
-  cur_right = AB(B, 4);
-  is_right = (Bool) AB(B, 5);
-
-  if (is_right)
-    {
-      cur_left = Pl_Range_Next_After(range, cur_left);
-      cur = cur_left;
-      is_right = FALSE;
-    }
-  else
-    {
-      cur_right = (cur_right >= 0) ? Pl_Range_Next_Before(range, cur_right)
-	: Max(fdv_adr);
-      cur = cur_right;
-      is_right = TRUE;
+      return FALSE;
     }
 
-  SYS_VAR_FD_BCKTS++;
-
-  if (cur != end)
+  if (Tag_Mask_Of(*fdv_adr) == TAG_INT_MASK)
     {
-#if 0 /* the following data is unchanged */
-      AB(B, 0) = (WamWord) fdv_adr;
-      AB(B, 1) = (WamWord) range;
-      AB(B, 2) = end;
-#endif
-      AB(B, 3) = cur_left;
-      AB(B, 4) = cur_right;
-      AB(B, 5) = is_right;
-    }
-  else if (Extra_Cstr(fdv_adr))
-    ALTB(B) = (CodePtr) Prolog_Predicate(EXTRA_CSTR_ALT, 0);
-  else
-    Delete_Last_Choice_Point();
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * INDOMAIN_RANDOM                                                         *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-static Bool
-Indomain_Random(WamWord *fdv_adr)
-{
-  Vector vec;
-  int n, cur;
-
-  if (!Pl_Fd_Use_Vector(fdv_adr))
-    return FALSE;
-
-  n = Nb_Elem(fdv_adr);
-
-  A(0) = (WamWord) fdv_adr;
-  A(1) = n;
-
-  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_RANDOM_ALT, 0),
-		      2 + pl_vec_size);
-
-  vec = (Vector) &AB(B, 2 - 1 + pl_vec_size);
-
-  Pl_Vector_Copy(vec, Vec(fdv_adr));
-
-  cur = Pl_Vector_Ith_Elem(vec, Pl_M_Random_Integer(n) + 1);
-  Vector_Reset_Value(vec, cur);
-
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_INDOMAIN_RANDOM_ALT_0                                                *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-Bool
-Pl_Indomain_Random_Alt_0(void)
-{
-  WamWord *fdv_adr;
-  Vector vec;
-  int n, cur;
-
-  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_RANDOM_ALT, 0),
-		      0);
-
-  fdv_adr = (WamWord *) AB(B, 0);
-  n = AB(B, 1);
-  vec = (Vector) (&AB(B, 2 - 1 + pl_vec_size));
-
-  SYS_VAR_FD_BCKTS++;
-  n--;
-
-  if (n > 1)
-    {
-#if 0 /* the following data is unchanged */
-      AB(B, 0) = (WamWord) fdv_adr;
-#endif
-      AB(B, 1) = n;
-#if 0 /* the following data is changed below */
-      AB(B,2...2-1+vecsize) = ...
-#endif
+      if (extra_cstr)
+	{
+	  /* A(0) = fdv_adr; */
+	  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(EXTRA_CSTR_ALT, 0), 1);
 	}
-  else if (Extra_Cstr(fdv_adr))
-    ALTB(B) = (CodePtr) Prolog_Predicate(EXTRA_CSTR_ALT, 0);
-  else
-    Delete_Last_Choice_Point();
+      return TRUE;
+    }
 
-  cur = Pl_Vector_Ith_Elem(vec, Pl_M_Random_Integer(n) + 1);
-  Vector_Reset_Value(vec, cur);
+  value = Select_Value(fdv_adr, value_method);
 
-  return Pl_Fd_Assign_Value(fdv_adr, cur);
+
+  /*  A(0) = (WamWord) fdv_adr | Extra_Cstr(fdv_adr); */
+  A(1) = value_method;		/* can change for METHOD_LIMITS */
+  A(2) = value;
+
+  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(INDOMAIN_ALT, 0), 3);
+
+  return Pl_Fd_Assign_Value_Fast(fdv_adr, value);
 }
 
 
@@ -824,7 +534,7 @@ Pl_Fd_Sel_Array_From_List_2(WamWord list_word, WamWord sel_array_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
-			WamWord reorder_word, WamWord fdv_word)
+			   WamWord reorder_word, WamWord fdv_word)
 #if 1
 #define PACK_ARRAY
 #endif
