@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.c,v 1.26 2012/05/31 08:14:17 diaz Exp $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -728,7 +728,7 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
   else
     DBGPRINTF("            ");
 
-  Pl_Write_Simple(inst_word);
+  Pl_Write(inst_word);
   DBGPRINTF("\n");
 #endif
 }
