@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_c.c,v 1.35 2012/11/14 16:21:47 diaz Exp $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -104,7 +104,7 @@ static int atom_atom_no_escape;
 static int atom_the_dialect;
 static int atom_the_home;
 
-static int atom_prolog[11];
+static int atom_prolog[15];
 
 
 
@@ -166,6 +166,12 @@ Flag_Initializer(void)
   atom_flag_tbl[FLAG_HOST_CPU] = Pl_Create_Atom("host_cpu");
   atom_flag_tbl[FLAG_HOST] = Pl_Create_Atom("host");
   atom_flag_tbl[FLAG_ARCH] = Pl_Create_Atom("arch");
+  atom_flag_tbl[FLAG_ADDRESS_BITS] = Pl_Create_Atom("address_bits");
+
+  atom_flag_tbl[FLAG_COMPILED_AT] = Pl_Create_Atom("compiled_at");
+  atom_flag_tbl[FLAG_C_CC] = Pl_Create_Atom("c_cc");
+  atom_flag_tbl[FLAG_C_CFLAGS] = Pl_Create_Atom("c_cflags");
+  atom_flag_tbl[FLAG_C_LDFLAGS] = Pl_Create_Atom("c_ldflags");
 
   atom_flag_tbl[FLAG_VERSION] = Pl_Create_Atom("version");
   atom_flag_tbl[FLAG_VERSION_DATA] = Pl_Create_Atom("version_data");
@@ -204,6 +210,15 @@ Flag_Initializer(void)
   atom_prolog[8] = Pl_Create_Atom(M_OS);
   atom_prolog[9] = Pl_Create_Atom(M_CPU "-" M_VENDOR "-" M_OS);
   atom_prolog[10] = Pl_Create_Atom(M_CPU "-" M_OS);
+#if defined(__DATE__) && defined(__TIME__)
+  atom_prolog[11] = Pl_Create_Atom(__DATE__ ", " __TIME__);
+#else
+  atom_prolog[11] = Pl_Create_Atom("unknown date");
+#endif
+  atom_prolog[12] = Pl_Create_Atom(CC);
+  atom_prolog[13] = Pl_Create_Atom(CFLAGS_MACHINE " " CFLAGS);
+  atom_prolog[14] = Pl_Create_Atom(LDFLAGS);
+ 
 
   Flag_Value(FLAG_SINGLETON_WARNING) = 1;
   Flag_Value(FLAG_SUSPICIOUS_WARNING) = 1;
@@ -218,11 +233,7 @@ Flag_Initializer(void)
    */
   Flag_Value(FLAG_BACK_QUOTES) = FLAG_AS_ATOM | FLAG_NO_ESCAPE_MASK; 
 
-#ifndef NO_USE_LINEDIT
-  SYS_VAR_LINEDIT = 1;
-#else
-  SYS_VAR_LINEDIT = 0;
-#endif
+  SYS_VAR_LINEDIT = pl_stream_use_linedit;
 }
 
 
@@ -272,6 +283,7 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
     case FLAG_MAX_ATOM:
     case FLAG_MAX_UNGET:
     case FLAG_VERSION:
+    case FLAG_ADDRESS_BITS:
       if (tag_mask != TAG_INT_MASK)
 	goto err_value;
       goto err_perm;
@@ -359,6 +371,10 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
     case FLAG_HOST_CPU:
     case FLAG_HOST:
     case FLAG_ARCH:
+    case FLAG_COMPILED_AT:
+    case FLAG_C_CC:
+    case FLAG_C_CFLAGS:
+    case FLAG_C_LDFLAGS:
       if (tag_mask != TAG_ATM_MASK)
 	goto err_value;
       goto err_perm;
@@ -582,9 +598,17 @@ Unif_Flag(int i, WamWord value_word)
     case FLAG_HOST_VENDOR:
     case FLAG_HOST:
     case FLAG_ARCH:
+    case FLAG_COMPILED_AT:
+    case FLAG_C_CC:
+    case FLAG_C_CFLAGS:
+    case FLAG_C_LDFLAGS:
       atom = atom_prolog[i - FLAG_PROLOG_NAME];
       break;
 
+    case FLAG_ADDRESS_BITS:
+      n = WORD_SIZE;
+      break;
+
     case FLAG_VERSION:
       n = __GPROLOG_VERSION__;
       break;
