@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_supp.h,v 1.24 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id$ */
 
 /*---------------------------------*
  * Constants                       *
@@ -75,6 +75,12 @@ enum
   FLAG_HOST_OS,
   FLAG_HOST,
   FLAG_ARCH,
+  FLAG_COMPILED_AT,
+  FLAG_C_CC,
+  FLAG_C_CFLAGS,
+  FLAG_C_LDFLAGS,
+
+  FLAG_ADDRESS_BITS,
 
   FLAG_VERSION_DATA,
   FLAG_VERSION,
