@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: foreign_supp.c,v 1.23 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id$ */
 
 #include <string.h>
 
@@ -95,11 +95,14 @@ static CodePtr Prepare_Call(int func, int arity, WamWord *arg_adr);
 
 
 #define CALL_INTERNAL              X1_2463616C6C5F696E7465726E616C
+#define THROW_INTERNAL             X1_247468726F775F696E7465726E616C
 
 #define PL_QUERY_RECOVER_ALT       X1_24706C5F71756572795F7265636F7665725F616C74
 
-Prolog_Prototype(PL_QUERY_RECOVER_ALT, 0);
 Prolog_Prototype(CALL_INTERNAL, 2);
+Prolog_Prototype(THROW_INTERNAL, 2);
+
+Prolog_Prototype(PL_QUERY_RECOVER_ALT, 0);
 
 
 
@@ -291,6 +294,25 @@ Pl_Exec_Continuation(int func, int arity, WamWord *arg_adr)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_THROW                                                                *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Throw(WamWord ball_word)
+{
+  int bip_func, bip_arity;
+
+  bip_func = Pl_Get_Current_Bip(&bip_arity);
+
+  A(0) = ball_word;
+  A(1) = Tag_INT(Call_Info(bip_func, bip_arity, 0));
+  Pl_Execute_A_Continuation(Prolog_Predicate(THROW_INTERNAL, 2));
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_QUERY_BEGIN                                                          *
  *                                                                         *
@@ -326,6 +348,20 @@ Pl_Query_Call(int func, int arity, WamWord *arg_adr)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_QUERY_START                                                          *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Query_Start(int func, int arity, WamWord *arg_adr, Bool recoverable)
+{
+  Pl_Query_Begin(recoverable);
+  return  Pl_Query_Call(func, arity, arg_adr);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_QUERY_RECOVER_ALT_0                                                  *
  *                                                                         *
@@ -592,7 +628,7 @@ Pl_Unif(PlTerm term1, PlTerm term2)
 
 
 /*-------------------------------------------------------------------------*
- * PL_Unif_With_Occurs_Check                                               *
+ * PL_UNIF_WITH_OCCURS_CHECK                                               *
  *                                                                         *
  * do not use directly Pl_Unify_Occurs_Check because of FC (fast call)     *
  *-------------------------------------------------------------------------*/
