@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: foreign_supp.h,v 1.18 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id$ */
 
 /*---------------------------------*
  * Constants                       *
@@ -108,26 +108,27 @@ void Pl_Foreign_Update_Choice(CodePtr codep_alt, int arity, int choice_size);
 CodePtr Pl_Foreign_Jump_Ret(CodePtr codep);
 
 PlFIOArg *Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word,
-			  PlLong (*rd_fct) (), int fio_arg_index);
+			       PlLong (*rd_fct) (), int fio_arg_index);
 
 Bool Pl_Foreign_Un_IO_Arg(int arg_long, Bool (*un_fct) (), PlFIOArg *fa,
-		       WamWord start_word);
+			  WamWord start_word);
 
 
 
 void Pl_Emit_Syntax_Error(char *file_name, int err_line, int err_col,
-		       char *err_msg);
+			  char *err_msg);
 
 
 
 void Pl_Exec_Continuation(int func, int arity, WamWord *arg_adr);
 
+void Pl_Throw(WamWord ball_word);
+
 void Pl_Query_Begin(Bool recoverable);
 
 int Pl_Query_Call(int func, int arity, WamWord *arg_adr);
 
-#define Pl_Query_Start(func, arity, arg_adr, recoverable) \
- (Pl_Query_Begin(recoverable), Pl_Query_Call(func, arity, arg_adr))
+int Pl_Query_Start(int func, int arity, WamWord *arg_adr, Bool recoverable);
 
 int Pl_Query_Next_Solution(void);
 
