@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: read.pl,v 1.21 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id$ */
 
 :-	built_in.
 
@@ -93,7 +93,7 @@ read_term(SorA, Term, Options) :-
 
 
 
-'$set_read_defaults' :-
+'$set_read_defaults' :-		%if modified, modified also const_io_c.c
 	'$sys_var_write'(0, 0),                                % default mask
 	'$sys_var_write'(1, -1).
 
