@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_supp.c,v 1.32 2012/11/13 14:00:55 diaz Exp $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -228,10 +228,10 @@ Init_Stream_Supp(void)
   pl_use_le_prompt = TRUE;
 
   pl_stm_stdin = Pl_Add_Stream_For_Stdio_Desc(stdin, pl_atom_user_input,
-					STREAM_MODE_READ, TRUE);
+					      STREAM_MODE_READ, TRUE);
 
 #ifndef NO_USE_LINEDIT
-  if ((pl_use_gui) || isatty(0))
+  if (pl_use_gui || isatty(0))
     {
       pstm = pl_stm_tbl[pl_stm_stdin];
       pstm->fct_getc = (StmFct) TTY_Getc;
@@ -249,7 +249,7 @@ Init_Stream_Supp(void)
 
 
   pl_stm_stdout = Pl_Add_Stream_For_Stdio_Desc(stdout, pl_atom_user_output,
-					 STREAM_MODE_WRITE, TRUE);
+					       STREAM_MODE_WRITE, TRUE);
 
 #if !defined(NO_USE_LINEDIT) && defined(_WIN32)
 		/* ok for both GUI and console EOM<->ANSI conversion */
@@ -267,7 +267,7 @@ Init_Stream_Supp(void)
 
 
   pl_stm_stderr = Pl_Add_Stream_For_Stdio_Desc(stderr, pl_atom_user_error,
-					 STREAM_MODE_WRITE, TRUE);
+					       STREAM_MODE_WRITE, TRUE);
 
 #if !defined(NO_USE_LINEDIT) && defined(_WIN32)
 		/* ok for both GUI and console EOM<->ANSI conversion */
@@ -363,7 +363,7 @@ Pl_Add_Stream_For_Stdio_Desc(FILE *f, int atom_path, int mode, int text)
     prop.eof_action = STREAM_EOF_ACTION_RESET;
 
   return Pl_Add_Stream(atom_path, (PlLong) f, prop,
-		    NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+		       NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 
@@ -1440,7 +1440,7 @@ Pl_Stream_Gets(char *str, int size, StmInf *pstm)
 
 
 /*-------------------------------------------------------------------------*
- * STREAM_GET_LINE                                                         *
+ * PL_STREAM_GETS_PROMPT                                                   *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 char *
@@ -1839,7 +1839,7 @@ Pl_Add_Str_Stream(char *buff, int prop_other)
 
 
 /*-------------------------------------------------------------------------*
- * REMOVE_STR_STREAM                                                       *
+ * PL_DELETE_STR_STREAM                                                    *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
