@@ -209,3 +209,24 @@ int sdl_init(PlTerm list)
   return PL_TRUE;
 }
 #endif
+
+
+
+#if 1
+#include "gprolog.h"
+
+
+
+Bool
+My_Disp(PlTerm term, PlTerm t1)
+{
+  char *s = Pl_Write_To_String(term);
+  
+  return Pl_Unif(Pl_Read_From_String(s), t1);
+  /*
+  printf("my <%s>\n", s);
+  free(s);
+  */
+}
+
+#endif
