@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_level.pl,v 1.29 2012/05/03 16:33:01 diaz Exp $ */
+/* $Id$ */
 
 :-	built_in.
 
@@ -46,7 +46,11 @@ top_level :-
 	current_prolog_flag(prolog_name, Name),
 	current_prolog_flag(prolog_version, Version),
 	current_prolog_flag(prolog_copyright, Copyright),
-	format(top_level_output, '~N~a ~a~n', [Name, Version]),
+	current_prolog_flag(address_bits, Bits),
+	current_prolog_flag(compiled_at, Date),
+	current_prolog_flag(c_cc, CC),
+	format(top_level_output, '~N~a ~a (~d bits)~n', [Name, Version, Bits]),
+	format(top_level_output, 'Compiled ~a with ~a~n', [Date, CC]),
 	write(top_level_output, 'By Daniel Diaz'),
 	nl,
 	format(top_level_output, '~a~n', [Copyright]),
