@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write_supp.c,v 1.33 2012/04/30 16:11:48 diaz Exp $ */
+/* $Id$ */
 
 #include <string.h>
 #include <ctype.h>
@@ -202,7 +202,7 @@ Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord *above_H,
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Write_Simple(WamWord term_word)
+Pl_Write(WamWord term_word)
 {
   StmInf *pstm = pl_stm_tbl[pl_stm_output];
 
