@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write_supp.h,v 1.20 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id$ */
 
 /*---------------------------------*
  * Constants                       *
@@ -79,7 +79,7 @@ extern int pl_last_writing;
 void Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord *above_H,
 		   WamWord term_word);
 
-void Pl_Write_Simple(WamWord term_word);
+void Pl_Write(WamWord term_word);
 
 void Pl_Write_A_Full_Stop(StmInf *pstm);
 
