@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_inst.c,v 1.25 2012/11/13 14:00:55 diaz Exp $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -78,9 +78,19 @@ static WamWord dummy_fd_var[FD_VARIABLE_FRAME_SIZE];
 
 static PlULong DATE;   /* NB: PlLong/PlULong have the same size as a WamWord (intptr_t) */
 
+/*
+ * When a constraint X in ...  is added the following sequence is executed:
+ * Pl_Fd_Before_Add_Cstr  (init the queue)
+ * compute the range/interval to restrict the variable X
+ * Pl_Fd_Tell_...(X, range/interval)   (restric X)
+ *    these functions call All_Propagations to add constraint depending on X to the queue
+ * Pl_Fd_After_Add_Cstr   (execute constraints in the queue)
+ *
+ * NB: Pl_Fd_Tell_... functions must be called in Pl_Fd_Before_Add_Cstr / Pl_Fd_After_Add_Cstr
+ */
 
 /*
- * FD_INT_Date(fdv_adr): tells at which date a FD var has been instantiated
+ * FD_INT_Date(fdv_adr): is the date at which the FD var has been instantiated
  * Optim #2: if a var has been instantiated before the post of the current 
  * constraint it is not necessary to reexecute it (in the propagation phase).
  *
@@ -113,16 +123,19 @@ static PlULong DATE;   /* NB: PlLong/PlULong have the same size as a WamWord (in
  * (which is the mask of all chains to reexecute for this var).
  * if Queue_Propag_Mask == 0 the var is not in the queue (else it is).
  *
- * Thus when a constraint is added, it is mandatory that all constraints 
- * are initially not marked. For this, Pl_Fd_Before_Add_Cstr() clears the 
- * variables remaining in the queue (i.e. of the previous constraint post).
- * This occurs if the previous constraint failed in the propagation phase
- * see Pl_Fd_After_Add_Cstr(). NB: in case of success, the queue has been
- * fully scanned and all variable are unmarked (no longer in the queue).
+ * Whan a constraint is told, in case of success, the queue has been
+ * fully scanned and all variable are unmarked (i.e. no longer in the queue).
+ * However, when a failure occurs in the propagation phase, some vars remain
+ * in the queue (marked). Thus Clear_Queue() is called to clean the queue.
+ *
+ *   NB: in 1.4.2, Clear_Queue() was called in Pl_Fd_Before_Add_Cstr() to clear 
+ *   the variables remaining in the queue (i.e. of the previous constraint post).
+ *   But this does not work if FD vars are created/restored (choice-point)
+ *   between a failure (remaining vars in the queue) and the next Clear_Queue().
  *
  * About propagation phase (Pl_Fd_After_Add_Cstr). The queue of constraint
  * having constraints to reconsider (reexecute) is handled as follows:
- * Queue_Next_Fdv_Adr(dummy_fd_var) points to the first variable
+ * Queue_Next_Fdv_Adr(dummy_fd_var) points to the first variable.
  * TP points to the last variable in queue
  * 
  * The queue is empty if TP == dummy_fd_var. Constraints are added at the
@@ -221,7 +234,7 @@ static void Clear_Queue(void);
       if (Range_Stamp(fdv_adr) != STAMP)			\
 	{							\
 	  Trail_MV(fdv_adr + OFFSET_RANGE, RANGE_SIZE);		\
-	  if (Is_Sparse(Range (fdv_adr)))			\
+	  if (Is_Sparse(Range(fdv_adr)))			\
 	      Trail_MV((WamWord *) Vec(fdv_adr), pl_vec_size);	\
 								\
 	  Range_Stamp(fdv_adr) = STAMP;				\
@@ -311,19 +324,6 @@ static void Clear_Queue(void);
 
 
 
-#define FD_Word_Needs_Trailing(adr)  ((adr) <  CSB(B))
-
-
-
-
-#define FD_Bind_OV(adr, word)       		\
-  do						\
-    {						\
-      if (FD_Word_Needs_Trailing(adr))		\
-	Trail_OV(adr);				\
-      *(adr) = (word);				\
-    }						\
-  while (0)
 
 
 
@@ -331,7 +331,7 @@ static void Clear_Queue(void);
 #define Update_Range_From_Range(fdv_adr, nb_elem, range, propag)           \
   do									   \
     {									   \
-      Range *r = Range (fdv_adr);					   \
+      Range *r = Range(fdv_adr);					   \
 									   \
       propag = MASK_EMPTY;						   \
 									   \
@@ -369,8 +369,7 @@ static void Clear_Queue(void);
 
 #ifdef DEBUG_CHECK_DATES_AND_QUEUE
 
-static WamWord *last_bp = dummy_fd_var;
-static WamWord *last_tp = dummy_fd_var;
+static WamWord *last_fdv_avr = NULL; /* a list of all FD vars (see Pl_Fd_New_Variable) */
 
 static void Check_Queue_Consistency(void);
 
@@ -466,11 +465,10 @@ Pl_Fd_Prolog_To_Fd_Var(WamWord arg_word, Bool pl_var_ok)
   if (tag_mask == TAG_INT_MASK)
     return Pl_Fd_New_Int_Variable(UnTag_INT(word));
 
-  if (tag_mask == TAG_FDV_MASK)
-    return UnTag_FDV(word);
+  if (tag_mask != TAG_FDV_MASK)
+    Pl_Err_Type(pl_type_fd_variable, word);
 
-  Pl_Err_Type(pl_type_fd_variable, word);
-  return NULL;
+  return UnTag_FDV(word);
 }
 
 
@@ -505,17 +503,16 @@ Pl_Fd_Prolog_To_Range(WamWord list_word)
 int
 Pl_Fd_Prolog_To_Value(WamWord arg_word)
 {
-  WamWord word, tag_mask;
+  PlLong v = Pl_Rd_Integer_Check(arg_word);
 
-  DEREF(arg_word, word, tag_mask);
+				/* conversion PlLong -> int (Fd only uses int) */
+  if (v < -INTERVAL_MAX_INTEGER)
+    v = -INTERVAL_MAX_INTEGER;
 
-  if (tag_mask == TAG_REF_MASK)
-    Pl_Err_Instantiation();
+  if (v > INTERVAL_MAX_INTEGER)
+    v = INTERVAL_MAX_INTEGER;
 
-  if (tag_mask != TAG_INT_MASK)
-    Pl_Err_Type(pl_type_integer, word);
-
-  return UnTag_INT(word);
+  return (int) v;
 }
 
 
@@ -555,15 +552,8 @@ Pl_Fd_List_Int_To_Range(Range *range, WamWord list_word)
 	Pl_Err_Type(pl_type_list, save_list_word);
 
       lst_adr = UnTag_LST(word);
-      DEREF(Car(lst_adr), word, tag_mask);
-      if (tag_mask == TAG_REF_MASK)
-	Pl_Err_Instantiation();
-
-      if (tag_mask != TAG_INT_MASK)
-	Pl_Err_Type(pl_type_integer, word);
-
-
-      val = UnTag_INT(word);
+      
+      val = Pl_Fd_Prolog_To_Value(Car(lst_adr));
 
       if ((unsigned) val > (unsigned) pl_vec_max_integer)
 	range->extra_cstr = TRUE;
@@ -844,13 +834,21 @@ Pl_Fd_Add_List_Dependency(WamWord *array, int chain_nb, WamWord *CF)
 
 
 /*-------------------------------------------------------------------------*
- * PL_FD_NEW_VARIABLE                                                      *
+ * PL_FD_NEW_VARIABLE_INTERVAL                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 WamWord *
-Pl_Fd_New_Variable(void)
+Pl_Fd_New_Variable_Interval(int min, int max)
 {
-  WamWord *fdv_adr = CS;
+  WamWord *fdv_adr;
+
+#ifdef DEBUG_CHECK_DATES_AND_QUEUE
+  Trail_OV(&last_fdv_avr);	/* reserve a cell just before the FD var to link prev FD var (stack) */
+  *CS = (WamWord) last_fdv_avr;
+  last_fdv_avr = CS++;
+#endif
+
+  fdv_adr = CS;
 
   FD_Tag_Value(fdv_adr) = Tag_FDV(fdv_adr);
   FD_INT_Date(fdv_adr) = DATE_ALWAYS;	/* must be awoken as long as tag == FDV */
@@ -859,8 +857,8 @@ Pl_Fd_New_Variable(void)
   Queue_Next_Fdv_Adr(fdv_adr) = NULL;
 
   Range_Stamp(fdv_adr) = STAMP;
-  Nb_Elem(fdv_adr) = INTERVAL_MAX_INTEGER + 1;
-  Range_Init_Interval(Range(fdv_adr), 0, INTERVAL_MAX_INTEGER);
+  Nb_Elem(fdv_adr) = max - min + 1;
+  Range_Init_Interval(Range(fdv_adr), min, max);
 
   Chains_Stamp(fdv_adr) = STAMP;
   Nb_Cstr(fdv_adr) = 0;
@@ -872,22 +870,30 @@ Pl_Fd_New_Variable(void)
   return fdv_adr;
 }
 
+/*-------------------------------------------------------------------------*
+ * PL_FD_NEW_VARIABLE                                                      *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+WamWord *
+Pl_Fd_New_Variable(void)
+{
+  return Pl_Fd_New_Variable_Interval(0, INTERVAL_MAX_INTEGER);
+}
+
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_FD_NEW_BOOL_VARIABLE                                                 *
+ * PL_FD_NEW_VARIABLE_RANGE                                                *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 WamWord *
-Pl_Fd_New_Bool_Variable(void)
+Pl_Fd_New_Variable_Range(Range *r)
 {
   WamWord *fdv_adr = Pl_Fd_New_Variable();
-
-  Nb_Elem(fdv_adr) = 2;
-  Min(fdv_adr) = 0;
-  Max(fdv_adr) = 1;
-
+  Pl_Range_Copy(Range(fdv_adr), r);
+  Nb_Elem(fdv_adr) = Pl_Range_Nb_Elem(r);
+ 
   return fdv_adr;
 }
 
@@ -946,11 +952,11 @@ Pl_Fd_Before_Add_Cstr(void)
   if (DATE == DATE_NEVER) /* reserve DATE_NEVER (i.e. 0) */
     DATE++;		  /* NB: it is not a problem if DATE == DATE_ALWAYS (i.e. 1) */
 
-  Clear_Queue();
+  TP = dummy_fd_var;		/* the queue is empty */
 
 #ifdef DEBUG_CHECK_DATES_AND_QUEUE
   Check_Queue_Consistency();
-#endif  
+#endif
 }
 
 
@@ -965,22 +971,20 @@ Pl_Fd_Before_Add_Cstr(void)
 static void
 Check_Queue_Consistency(void)
 {
-  WamWord *BP = last_bp;
-  WamWord *fdv_adr;
+  WamWord *fdv_adr = last_fdv_avr;
+  WamWord *prev;
 
-  if (last_tp == dummy_fd_var)	/* empty ? */
-    return;
-
-  for(;;) 
+  while(fdv_adr != NULL)
     {
-      fdv_adr = (WamWord *) BP;
+      prev = (WamWord *) (*fdv_adr); /* link to (cell - 1 of) previous FD var */
+      fdv_adr++;	      /* FD var is just after stack previous links */
+#if 0
+      printf("Checking var:_%ld (%p)\n", Cstr_Offset(fdv_adr), fdv_adr);
+#endif
       if (Is_Var_In_Queue(fdv_adr))
 	printf("ERROR QUEUE should be empty but contains var:_%ld (%p)\n", Cstr_Offset(fdv_adr), fdv_adr);
 
-      if (BP == last_tp)
-	break;
-		    
-      BP = Queue_Next_Fdv_Adr(BP);
+      fdv_adr = prev;
     }
 }
 
@@ -1264,6 +1268,53 @@ Pl_Fd_Tell_Range_Range(WamWord *fdv_adr, Range *range)
 
 
 
+
+/*-------------------------------------------------------------------------*
+ * PL_FD_TELL_INTERVAL                                                     *
+ *                                                                         *
+ * Called by fd_to_c.h                                                     *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_Tell_Interval(WamWord *fdv_adr, int min, int max)
+{
+  int n;
+  Range range;
+
+  if (Fd_Variable_Is_Ground(fdv_adr))
+    {
+      n = Min(fdv_adr);
+      return (n >= min && n <= max); /* also detects if initial range is empty */
+    }
+
+  if (Is_Sparse(Range(fdv_adr)))
+    {
+      Range_Init_Interval(&range, min, max);
+      return Pl_Fd_Tell_Range_Range(fdv_adr, &range);
+    }
+
+  return Pl_Fd_Tell_Interv_Interv(fdv_adr, min, max);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_FD_TELL_RANGE                                                        *
+ *                                                                         *
+ * Called by fd_to_c.h                                                     *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_Tell_Range(WamWord *fdv_adr, Range *range)
+{
+  if (Fd_Variable_Is_Ground(fdv_adr))
+    return Pl_Fd_Tell_Int_Range(fdv_adr, range);
+
+  return Pl_Fd_Tell_Range_Range(fdv_adr, range);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * ALL_PROPAGATIONS                                                        *
  *                                                                         *
@@ -1305,20 +1356,11 @@ Pl_Fd_After_Add_Cstr(void)
   WamWord *AF;
   PlLong (*fct) ();
 
-#ifdef DEBUG_CHECK_DATES_AND_QUEUE
-  last_bp = NULL;
-  last_tp = TP;
-#endif
-
   if (TP == dummy_fd_var)
     return TRUE;
 
   BP = Queue_Next_Fdv_Adr(dummy_fd_var);
 
-#ifdef DEBUG_CHECK_DATES_AND_QUEUE
-  last_bp = BP;
-#endif
-
   for (;;)
     {
       fdv_adr = (WamWord *) BP;
@@ -1355,6 +1397,7 @@ Pl_Fd_After_Add_Cstr(void)
 		  {
 		  failure:
 		    Queue_Next_Fdv_Adr(dummy_fd_var) = BP; /* update begin of remaining queue */
+		    Clear_Queue(); /* Do it now, not in Pl_Fd_Before_Add_Cstr */
 		    return FALSE;
 		  }
 #if 1				/* FD switch */
@@ -1409,15 +1452,47 @@ Pl_Fd_Stop_Constraint(WamWord *CF)
 
 
 /*-------------------------------------------------------------------------*
- * PL_FD_ASSIGN_VALUE                                                      *
+ * PL_FD_IN_INTERVAL                                                       *
  *                                                                         *
- * fdv_adr is a FDV and n belongs to the range of the FD var               *
- * like Pl_Fd_Unify_With_Integer0() but specialized Fd_Tell_Value without  *
+ * Used by domain predicates.                                              *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_In_Interval(WamWord *fdv_adr, int min, int max)
+{
+  Pl_Fd_Before_Add_Cstr();
+
+  return Pl_Fd_Tell_Interval(fdv_adr, min, max) && Pl_Fd_After_Add_Cstr();
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_FD_IN_RANGE                                                          *
+ *                                                                         *
+ * Used by domain predicates.                                              *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_In_Range(WamWord *fdv_adr, Range *range)
+{
+  Pl_Fd_Before_Add_Cstr();
+
+  return Pl_Fd_Tell_Range(fdv_adr, range) && Pl_Fd_After_Add_Cstr();
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_FD_ASSIGN_VALUE_FAST                                                 *
+ *                                                                         *
+ * fdv_adr is an FDV and n belongs to the range of the FD var.             *
+ * Like Pl_Fd_Unify_With_Integer0 but specialized Pl_Fd_Tell_Value without *
  * useless tests (ie. groundness and Pl_Range_Test_Value())                *
  * Used by labeling predicates.                                            *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Fd_Assign_Value(WamWord *fdv_adr, int n)
+Pl_Fd_Assign_Value_Fast(WamWord *fdv_adr, int n)
 {
   int propag;
 
@@ -1448,6 +1523,21 @@ Pl_Fd_Unify_With_Integer0(WamWord *fdv_adr, int n)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_FD_REMOVE_VALUE                                                      *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Fd_Remove_Value(WamWord *fdv_adr, int n)
+{
+  Pl_Fd_Before_Add_Cstr();
+
+  return Pl_Fd_Tell_Not_Value(fdv_adr, n) && Pl_Fd_After_Add_Cstr();
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_FD_UNIFY_WITH_FD_VAR0                                                *
  *                                                                         *
@@ -1485,7 +1575,7 @@ Pl_Fd_Use_Vector(WamWord *fdv_adr)
 
     Pl_Range_Becomes_Sparse(&range);
 
-    CS=save_CS;			/* code of fd_deallocate (from fd_to_c.h) */
+    CS = save_CS;		/* code of fd_deallocate (from fd_to_c.h) */
   }
   return Pl_Fd_Tell_Range_Range(fdv_adr, &range) && Pl_Fd_After_Add_Cstr();
 }
