@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ma_parser.c,v 1.25 2012/01/10 17:46:02 diaz Exp $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -410,7 +410,7 @@ Parser(int pass_no, int nb_passes)
 	    cur_line_p++;
 	  if (*cur_line_p != '=')
 	    {
-	      Decl_Long(name, global, NONE, 0);
+	      Decl_Long(name, global, NONE, 1); /* default: NONE as value = 1 (value = array size = 1) */
 	      break;
 	    }
 	  cur_line_p++;		/* skip the = */
