@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: decode_hexa.c,v 1.19 2012/01/10 17:46:02 diaz Exp $ */
+/* $Id$ */
 
 #include <string.h>
 #include <ctype.h>
@@ -236,8 +236,8 @@ Encode_Hexa_Line(char *str, char *format, int strict)
  * STRING_NEEDS_ENCODING                                                   *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static
-int String_Needs_Encoding(char *str)
+static int
+String_Needs_Encoding(char *str)
 {
   if (*str == '\0' || !Letter_Digit(*str))
     return 1;
