@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_comp.c,v 1.38 2012/11/20 09:15:46 diaz Exp $ */
+/* $Id$ */
 
 #include "../EnginePl/gp_config.h"
 
@@ -1064,9 +1064,9 @@ Parse_Arguments(int argc, char *argv[])
 	      continue;
 	    }
 
-	  if (Check_Arg(i, "--inline-asm") ||
-	      Check_Arg(i, "--full-inline-asm"))
-	    {
+	  if (Check_Arg(i, "--inline-asm") || Check_Arg(i, "--full-inline-asm") ||
+	      Check_Arg(i, "--pic") || Check_Arg(i, "-fPIC")) /* TODO pass --pic to gcc as -fPIC for C code */
+	    {		
 	      Add_Last_Option(cmd_ma2asm.opt);
 	      continue;
 	    }
@@ -1487,6 +1487,7 @@ Display_Help(void)
   L(" ");
   L("Mini-assembly to assembly translator options:");
   L("  --comment                   include comments in the output file");
+  L("  --pic                       produce position independent code (PIC)");
   L("  --inline-asm                inline some C calls as asm instructions");
   L("  --full-inline-asm           inline most C calls as asm instructions");
   L(" ");
